/*
  Copy EEPROM to microSD SDEEPROM

*/
#include <SPI.h>
#include <SD.h>
#include <EEPROM.h>
#include <LiquidCrystal_Wire2_I2C.h>
extern LiquidCrystal_Wire2_I2C lcd;//  indicate that "lcd" was already defined in <ubitx_V1_08_I2C>
extern void printLine2(char *c);


extern void printLine1(char *c);

extern int   Working_Copy_Modes[10];
extern unsigned long Working_Copy_Frequencies[10];
extern int TableMODES[4][10];
extern unsigned long TableFREQS[4][10];


void SD_OPERATION_FAILED(void) {

  lcd.clear();
  printLine1((char *)"SD DISK FAIL->EEPROM");
  while ( 1 == 1) delay(100);
}




bool Put_SDEEPROM( int index, long Value ) {
  File fileptr;
  byte Data[4];

  // open file SDEEPROM for writing
  fileptr = SD.open("SDEEPROM", FILE_WRITE);
  if ( fileptr ) {
    // move position to beginning of file
    fileptr.seek(index);
    Data[3] = Value & 0xff;
    Data[2] = (Value >> 8) & 0xff;
    Data[1] = (Value >> 16) & 0xff;
    Data[0] = (Value >> 24) & 0xff;
    fileptr.write(Data, 4);
    fileptr.close();
  } else SD_OPERATION_FAILED();
  return (true);

}




bool Backup_EEPROM_to_SD( void ) {
  int i;
  long temp;

  for (i = 0; i < 76; i = i + 4) {
    EEPROM.get(i, temp);
    if ( !Put_SDEEPROM(i, temp)) return (false);
  }


  return ( true );

}


long Get_SDEEPROM( int index ) {
  File fileptr;
  byte Data[4];
  long Value;

  // open file SDEEPROM for writing
  fileptr = SD.open("SDEEPROM", FILE_READ);
  if ( fileptr ) {
    fileptr.seek(index);
    fileptr.read(Data, 4);

    Value = Data[0] & 0xff;
    Value = (Value << 8) + Data[1];
    Value = (Value << 8) + Data[2];
    Value = (Value << 8) + Data[3];
    fileptr.close();

  } else {
    SD_OPERATION_FAILED();
  }
  return (Value);
}


/*
  void Print_SD_EEPROM( void ){
  int i;
  long temp;

    for (i=0;i<256;i=i+4){
      Serial.print(i);
      temp = Get_SDEEPROM(i);
      if( SD_OPERATION_FAILED ){
        Serial.println("Failed");
      }else{
        Serial.print(" = ");
        Serial.println(temp);
      }
    }
  }
*/

void Do_SD_Init( void ) {
  File fileptr;
  int i;

  if (!Backup_EEPROM_to_SD() ) {
    SD_OPERATION_FAILED();
  } else {
    for (i = 0; i < 10; i++) {
      Put_SDEEPROM( 100 + (i * 4), TableFREQS[0][i] );
      Put_SDEEPROM( 140 + (i * 4), TableMODES[0][i] );
    }
    for (i = 0; i < 10; i++) {
      Put_SDEEPROM( 200 + (i * 4), TableFREQS[1][i] );
      Put_SDEEPROM( 240 + (i * 4), TableMODES[1][i] );
    }
    for (i = 0; i < 10; i++) {
      Put_SDEEPROM( 300 + (i * 4), TableFREQS[2][i] );
      Put_SDEEPROM( 340 + (i * 4), TableMODES[2][i] );
    }
    for (i = 0; i < 10; i++) {
      Put_SDEEPROM( 400 + (i * 4), TableFREQS[3][i] );
      Put_SDEEPROM( 440 + (i * 4), TableMODES[3][i] );
    }
  }

}


void SD_Setup() {
  File fileptr;
  int i;


  if (!SD.begin(BUILTIN_SDCARD)) {
    SD_OPERATION_FAILED();
  } else {
    // open file SDEEPROM for writing
    fileptr = SD.open("SDEEPROM", FILE_WRITE);
    if ( fileptr ) {
      i = fileptr.size();
      if (i < 50 )Do_SD_Init();
      fileptr.close();
    } else {
      Do_SD_Init();
    }
  }
}

