
//   COFIGURATION PARAMETERS    //


// TO ENABLE NANO'S LED TO INDICATE WHEN TRANSMITING 
// UNCOMMENT THE NEXT 2 LINES
#define XMIT_LED 1
#define ON_AIR LED_BUILTIN






// Keyboard Modes
#define KEY_IDLE   0
#define KEY_FREQ   1
#define KEY_TONE   2
#define KEY_WPM    3
#define KEY_CW     4

// Keyboard Commands:
#define CTL_c   3    //CTL_c         Toggle CW Keyboard
#define CTL_f   6    //CTL_fxxxxxxxx set active vfo frequency
#define CTL_i   9    //CTL_i         toggle iambic 
#define CTL_m   13   //CTL_m         change mode
#define CTL_p   16   //CTL_p         toggle paddle
#define CTL_r   18   //CTL_r         toggle RIT 
#define CTL_s   19   //CTL_s         toggle SPLIT
#define CTL_t   20   //CTL_txxx      set CW tone       100=< xxx <= 990
#define CTL_v   22   //CTL_v         toggle VFO
#define CTL_w   23   //CTL_wxx       set CW speed      5 =< xx <= 50
#define CTL_IF  14   //CTL_>         toggle IF shift
/*

 *File name ubitx.h 
 *  The second set of 16 pins on the Raduino's bottom connector are have the three clock outputs and the digital lines to control the rig.
 */ 


/**
 * These are the indices where these user changeable settings are stored  in the EEPROM
 */
#define MASTER_CAL   0
#define LSB_CAL      4
#define USB_CAL      8
#define SIDE_TONE   12
//these are ID's of the VFO's as well as their offset into the EEPROM storage, don't change these 'magic' values
#define VFO_A       16
#define VFO_B       20
#define CW_SIDETONE 24
#define CW_SPEED    28


//  Added EEPROM values

#define FREQ_INCR   32
#define SELECTED_MEM_TABLE  36 
#define MODE        40
#define VOLTS       44
#define BAUD        48
#define XTALPF      52
#define IAMBIC_MODE 56
#define PDL_SWAP_VAL 60
#define VOLT_DIVIDE 64
#define SMETER_ACTIVE 68
#define CLOCK_ACTIVE 72
#define CW_FILTER_PRESENT 76
#define CW_FILTER_CALIBRATED 80
#define CW_FILTER_OFFSET 84


/**
 * The uBITX is an upconversion transceiver. The first IF is at 45 MHz.
 * The first IF frequency is not exactly at 45 MHz but about 5 khz lower,
 * this shift is due to the loading on the 45 MHz crystal filter by the matching
 * L-network used on it's either sides.
 * The first oscillator works between 48 MHz and 75 MHz. The signal is subtracted
 * from the first oscillator to arrive at 45 MHz IF. Thus, it is inverted : LSB becomes USB
 * and USB becomes LSB.
 * The second IF of 12 MHz has a ladder crystal filter. If a second oscillator is used at 
 * 57 MHz, the signal is subtracted FROM the oscillator, inverting a second time, and arrives 
 * at the 12 MHz ladder filter thus double inversion, keeps the sidebands as they originally were.
 * If the second oscillator is at 33 MHz, the oscillator is subtracted from the signal, 
 * thus keeping the signal's sidebands inverted. The USB will become LSB.
 * We use this technique to switch sidebands. This is to avoid placing the lsbCarrier close to
 * 12 MHz where its fifth harmonic beats with the Arduino's 16 MHz oscillator's fourth harmonic
 */

// the second oscillator should ideally be at 57 MHz, however, the crystal filter's center frequency 
// is shifted down a little due to the loading from the impedance matching L-networks on either sides
#define SECOND_OSC_USB (56995000l)
#define SECOND_OSC_LSB (32995000l) 
//these are the two default USB and LSB frequencies. The best frequencies depend upon your individual taste and filter shape
#define INIT_USB_FREQ   (11996500l)
// limits the tuning and working range of the uBITX between 3 MHz and 30 MHz
#define LOWEST_FREQ  (3000000l)
#define HIGHEST_FREQ (30000000l)

//we directly generate the CW by programming the Si5351 to the cw tx frequency, hence, both are different modes
//these are the parameters passed to startTx
#define TX_SSB 0
#define TX_CW 1

#define CW_TIMEOUT (350l) 
#define PTT_HNDKEY_DEBOUNCE_CT 2


#define Ubitx_Voltage_Timer_Count 3000

#define MODE_USB  0
#define MODE_LSB  1
#define MODE_CW   2
#define MODE_CWR  3
#define MODE_SWU  4
#define MODE_SWL  5
#define MODE_CWP  6


#define BUTTON_DELAY_1 40
#define BUTTON_DELAY_2 120
#define BUTTON_DELAY_3 260
#define BUTTON_DELAY_4 360

/////////////////////////  keyer

#define DIT_L 0x01 // DIT latch
#define DAH_L 0x02 // DAH latch
#define DIT_PROC 0x04 // DIT is being processed
enum KSTYPE {IDLE, CHK_DIT, CHK_DAH, KEYED_PREP, KEYED, INTER_ELEMENT };

//////////////////////////////////////////////


#define PRESS_QUICK  80
#define PRESS_MEDIUM 160
#define PRESS_LONG   240
#define PRESS_LONGER   320
#define PRESS_LONGEST  400
#define ENCODER_FREQ_SHIFT 20


/**
 * We need to carefully pick assignment of pin for various purposes.
 * There are two sets of completely programmable pins on the Raduino.
 * First, on the top of the board, in line with the LCD connector is an 8-pin connector
 * that is largely meant for analog inputs and front-panel control. It has a regulated 5v output,
 * ground and six pins. Each of these six pins can be individually programmed 
 * either as an analog input, a digital input or a digital output. 
 * The pins are assigned as follows (left to right, display facing you): 
 *      Pin 1 (Violet), A7, KEYER (DASH)
 *      Pin 2 (Blue),   A6, KEYER (DOT)
 *      Pin 3 (Green), +5v 
 *      Pin 4 (Yellow), GND
 *      Pin 5 (Orange), A3, PTT
 *      Pin 6 (Red),    A2, F BUTTON
 *      Pin 7 (Brown),  A1, ENC B
 *      Pin 8 (Black),  A0, ENC A
 *Note: A5, A4 are wired to the Si5351 as I2C interface 
 *       *     
 * Though, this can be assigned anyway, for this application of the Arduino, we will make the following
 * assignment
 * A2 will connect to the PTT line, which is the usually a part of the MIC connector
 * A3 is connected to a push button that can momentarily ground this line. This will be used for RIT/Bandswitching, etc.
 * A6 is connected to the DOT lead from the paddle
 * A7 is connected to the DASH lead from the paddle
  */
//////////////////////////////////////////////////////   PIN DEFINITONS12345789
//#define RADI2CINO

#ifdef RADI2CINO      // definitions for RadI2Cino Teensy Adapter Card

#define MENU_SEL_ENC_A (24)
#define MENU_SEL_ENC_B (25)
#define MENU_ENTER_BUTTON (26)
#define MENU_CHOOSE_BUTTON (27)
#define ENC_A (A1)
#define ENC_B (A0)
#define FBUTTON (A2)
#define UBIT_VCC (A6)
#define AB_SPLIT (10)
#define DIGITAL_DOT  (12)
#define DIGITAL_DASH (11)
#define DIGITAL_KEY (8)
#define PTT   (8)


#define TX_RX (7)
#define CW_TONE (6)
#define TX_LPF_A (5)
#define TX_LPF_B (4)
#define TX_LPF_C (3)
#define CW_KEY (2)



#else

  #define TX_RX (10)
  #define CW_TONE (9)
  #define TX_LPF_A (8)
  #define TX_LPF_B (7)
  #define TX_LPF_C (6)
  #define CW_KEY (5)




  #define MENU_SEL_ENC_A (26)
  #define MENU_SEL_ENC_B (27)
  #define MENU_ENTER_BUTTON (29)
  #define MENU_CHOOSE_BUTTON (30)
  #define ENC_A (19)
  #define ENC_B (18)
  #define FBUTTON (20)
  #define UBIT_VCC (A13)
  #define AB_SPLIT (28)
  #define DIGITAL_DOT  (22)
  #define DIGITAL_DASH (23)
  #define DIGITAL_KEY (21)
  #define PTT   (21)

#define MEMORY_TABLES_BUTTON (24)
#define MEMORY_CHANNEL_BUTTON (25)
#define DISPLAY_SEQUENCE_BUTTON (31)
  #define SMETERREADPIN (A19)
#endif


