// File name ubitx_Configuration.cpp


#include <Arduino.h>
#include <EEPROM.h>
#include <TimeLib.h>
#include "ubitx.h"
#include <LiquidCrystal_Wire2_I2C.h>
extern LiquidCrystal_Wire2_I2C lcd;


extern bool btnDown(void);
extern void printLine1(char *c);
extern void printLine2(char *c);
extern void printLine3(char *c);
extern void printLine4(char *c);
extern void printLineXI(char linenmbr, char index, char *c);
extern void si5351_set_calibration(int32_t cal);
extern void si5351bx_setfreq(uint8_t clknum, uint32_t fout);
extern void cwKeydown(void);
extern void cwKeyUp(void);
extern int  enc_read(void);
extern void startTx(byte txMode);
extern void stopTx(void);
extern void initOscillators(void);
extern void setFrequency(unsigned long f);
extern void updateDisplay(void);
extern void checkPTT(void);
extern void printLine(char linenmbr, char *c);
extern void ritDisable(void);
extern int  Menu_Select_Encoder_Read(void);
extern bool Backup_EEPROM_to_SD( void );
extern bool Put_SDEEPROM( int index, long Value );
extern long Get_SDEEPROM( int index );
extern bool Choose_Down(void);
extern int Menu_Select_Encoder_Read();
extern bool Enter_Exit_Down(void);
extern void digitalClockDisplay();

extern unsigned long usbCarrier;
extern bool swlUSB;
extern char c[21], b[21];
extern volatile bool keyDown;
extern unsigned long cwTimeout;
extern unsigned long frequency;
extern unsigned long Freq_Incr[6];
extern int Freq_Index;
extern volatile int ubitx_mode;
extern char ritOn;
extern char vfoActive;
extern int Freq_Underline[6];
extern unsigned int SI5351BX_XTALPF;              // 1:6pf  2:8pf  3:10pf
extern volatile unsigned long Ubitx_Voltage;
extern volatile unsigned long Current_Ubitx_Voltage;
extern volatile unsigned long Converted_Ubitx_Voltage;
extern unsigned long Volt_Divider;
extern bool Cat_Lock;
extern bool Volts_Active;
extern bool Smeter_Active;
extern bool Clock_Active;
extern char printBuff[4][21];
extern int TableMODES[4][10];
extern unsigned long TableFREQS[4][10];


bool CW_Filter_Present = false;
bool CW_Calibrated = false;
unsigned long CW_Filter_Offset = 0;
unsigned long Standard_Offset = 0;

int ubitx_baud = 5;
int Last_Baud = 5;
char ubitx_CAT_Baud_rates[8][7] = {"1200", "2400", "4800", "9600", "19200", "38400", "57600", "115200"};
long ubitx_CAT_Long_Baud_rates[8] = {1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200};

byte Config_Loop;

unsigned long Cal_Freq_Incr[7] = {1, 10, 100, 1000, 10000, 100000, 1000000};
int Cal_Freq_Underline[7] = {19, 18, 17, 15, 14, 13, 11};
int Cal_Freq_Index = 0;

void Blank_Screen( void) {

  lcd.clear();
  strcpy(printBuff[0], "");
  strcpy(printBuff[1], "");
  strcpy(printBuff[2], "");
  strcpy(printBuff[3], "");
  delay(100);
}

//returns true if the button is pressed
bool Menu_BTN_Down(void) {

  if (digitalRead(MENU_CHOOSE_BUTTON) == HIGH) return false;
  else return true;
}

/**
   This procedure is only for those who have a signal generator/transceiver tuned to exactly 7.150 and a dummy load
*/
char Pf_Text[4][12] = { "XTAL  6pf", "XTAL  6pf", "XTAL  8pf", "XTAL 10pf"};

void   Menu_Set_Pf( int btn ) {
  int s;
  char buf[21];

  SI5351BX_XTALPF = Get_SDEEPROM( XTALPF );

  if (!btn) {
    sprintf(buf, "OSC LOAD %s", Pf_Text[SI5351BX_XTALPF]);
    printLine4((char*)"   ");
    printLine1(buf);
    return;
  }

  if ( (SI5351BX_XTALPF < 1) || (SI5351BX_XTALPF > 3)) SI5351BX_XTALPF = 2;
  printLine2((char*)"Press Choose to set");
  lcd.setCursor(15, 0);
  while (Menu_BTN_Down())delay(10);
  while ( !Menu_BTN_Down() ) {
    s = Menu_Select_Encoder_Read();
    if (s != 0 ) {
      if (s > 1) SI5351BX_XTALPF = SI5351BX_XTALPF + 1;
      else if (s < -1) SI5351BX_XTALPF = SI5351BX_XTALPF - 1;

      if ( SI5351BX_XTALPF < 1 ) SI5351BX_XTALPF = 1;
      if ( SI5351BX_XTALPF > 3 ) SI5351BX_XTALPF = 3;
      sprintf(buf, "OSC LOAD %s", Pf_Text[SI5351BX_XTALPF]);
      printLine1(buf);
      lcd.setCursor(15, 0);
      delay(500);
    }
  }
  Put_SDEEPROM( XTALPF, SI5351BX_XTALPF );
  while ( Menu_BTN_Down())delay(10);
  printLine2((char*)" ");


}


//this is used by the si5351 routines in the ubitx_5351 file
extern int32_t calibration;
extern uint32_t si5351bx_vcoa;
float temp_value;

void Update_Freq(void) {
  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));

  strcpy(c, (char *)"Tuned:LSB ");
  ultoa(frequency, b, DEC);
  //one mhz digit if less than 10 M, two digits if more
  if (strlen(b) < 7) {
    strcat(&c[3], "00.");
    strncat(c, &b[0], 3);
    strcat(c, ".");
    strncat(c, &b[3], 3);
  } else if (strlen(b) < 8) {
    strcat(&c[6], "0");
    strncat(c, &b[0], 1);
    strcat(c, ".");
    strncat(c, &b[1], 3);
    strcat(c, ".");
    strncat(c, &b[4], 3);
  } else {
    strncat(c, b, 2);
    strcat(c, ".");
    strncat(c, &b[2], 3);
    strcat(c, ".");
    strncat(c, &b[5], 3);
  }


  printLine(3, c);
  lcd.setCursor(Cal_Freq_Underline[Cal_Freq_Index], 3);

}

void CaldoTuning() {
  int s;

  s = enc_read();
  if (s) {

    if (s > 4)
      frequency += Cal_Freq_Incr[Cal_Freq_Index];
    else if (s >  2)
      frequency += Cal_Freq_Incr[Cal_Freq_Index];
    else if (s >  0)
      frequency +=  Cal_Freq_Incr[Cal_Freq_Index];
    else if (s <  0)
      frequency -= Cal_Freq_Incr[Cal_Freq_Index];
    else if (s < -2)
      frequency -= Cal_Freq_Incr[Cal_Freq_Index];
    else if (s < -4)
      frequency -= Cal_Freq_Incr[Cal_Freq_Index];
    if ( frequency < 1000000) frequency = 1000000;

    setFrequency(frequency);
    Update_Freq();
  }
}



int menuSetupCalibration( void ) {
  int knob = 0;
  int32_t prev_calibration;



  prev_calibration = calibration;
  calibration = 0;
  si5351_set_calibration(calibration);
  setFrequency(frequency);

  printLine2((char *)"press PTT to save");
  strcpy(b, "Zero-beat cal:");
  ltoa(calibration / 8750, c, 10);
  strcat(b, c);
  printLine3(b);
  lcd.setCursor(14, 2);

  while (digitalRead(PTT) == HIGH && !btnDown())
  {
    knob = enc_read();

    if (knob > 0) {
      calibration += 8750;
      usbCarrier += 120;
    }
    else if (knob < 0) {
      calibration -= 8750;
      usbCarrier -= 120;
    }
    else
      continue; //don't update the frequency or the display

    si5351_set_calibration(calibration);
    si5351bx_setfreq(0, usbCarrier);
    setFrequency(frequency);

    strcpy(b, "Zero-beat cal:");
    ltoa(calibration / 8750, c, 10);
    strcat(b, c);
    printLine3(b);
    lcd.setCursor(14, 2);
  }

  //save the setting
  if (digitalRead(PTT) == LOW) {
    while ( digitalRead(PTT) == LOW ) delay(10);
    printLine1((char *)"Calibration set!");
    printLine2((char *)"Set Carrier now");
    Put_SDEEPROM( MASTER_CAL, calibration );
    // test to see if need to also save usbCarrier
    // comment out the next line if does not work better
    //Put_SDEEPROM( USB_CAL, usbCarrier );

    delay(2000);
  } else calibration = prev_calibration;

  printLine2((char *)"");
  initOscillators();
  //si5351_set_calibration(calibration);
  setFrequency(frequency);
  return (0);
}


void printCarrierFreq(unsigned long freq) {

  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));

  strcpy(c, "PTTconfirm");

  ultoa(freq, b, DEC);

  if (strlen(b) < 7) {
    strcat(&c[3], "00.");
    strncat(c, &b[0], 3);
    strcat(c, ".");
    strncat(c, &b[3], 3);
  } else if (strlen(b) < 8) {
    strcat(&c[6], "0");
    strncat(c, &b[0], 1);
    strcat(c, ".");
    strncat(c, &b[1], 3);
    strcat(c, ".");
    strncat(c, &b[4], 3);
  } else {
    strncat(c, b, 2);
    strcat(c, ".");
    strncat(c, &b[2], 3);
    strcat(c, ".");
    strncat(c, &b[5], 3);
  }
  printLineXI(1, 0, c);
  lcd.setCursor(Cal_Freq_Underline[Cal_Freq_Index], 1);
}

void menuSetupCarrier(void) {
  int knob = 0;
  unsigned long prevCarrier;

  //keep clear of any previous button press
  while (btnDown())
    delay(50);
  delay(50);

  prevCarrier = usbCarrier;
  printLine1((char *)"Step 3: Best Signal");

  usbCarrier = 11995000l;
  si5351bx_setfreq(0, usbCarrier);
  printCarrierFreq(usbCarrier);

  //disable all clock 1 and clock 2
  //while (digitalRead(PTT) == HIGH && !btnDown())
  while (digitalRead(PTT) == HIGH )
  {
    if ( btnDown()) {
      while (btnDown())delay(10);
      Cal_Freq_Index = Cal_Freq_Index - 1;
      if ( Cal_Freq_Index < 0 )Cal_Freq_Index = 6;
      printCarrierFreq(usbCarrier);
    }
    knob = enc_read();

    if (knob > 0) {
      usbCarrier += Cal_Freq_Incr[Cal_Freq_Index];
      printCarrierFreq(usbCarrier);
    } else if (knob < 0) {
      usbCarrier -= Cal_Freq_Incr[Cal_Freq_Index];
      printCarrierFreq(usbCarrier);
    } else
      continue; //don't update the frequency or the display

    si5351bx_setfreq(0, usbCarrier);

    delay(100);
  }

  //save the setting
  if (digitalRead(PTT) == LOW) {
    printLine2((char *)"Carrier set!    ");
    Put_SDEEPROM( USB_CAL, usbCarrier );
    delay(1000);
  }
  else
    usbCarrier = prevCarrier;

  si5351bx_setfreq(0, usbCarrier);
  setFrequency(frequency);
  updateDisplay();
  printLine2((char *)"     ");
}

void btnWaitForClick() {
  while (!btnDown())
    delay(50);
  while (btnDown())
    delay(50);
  delay(50);
}


void Menu_Calibrate( int btn ) {
  int i;
  bool looper = true;

  //keep clear of any previous button press
  while (btnDown())
    delay(50);
  delay(50);

  if (!btn) {
    printLine1((char *)"CALIBRATE");
    return;
  }
  printLine1((char *)"Step 1:Tune station");
  printLine2((char *)"Tune button Nxt Step");
  ubitx_mode = MODE_LSB;
  ritDisable();
  Update_Freq();

  while ( looper ) {
    //only if the button is pressed
    if (btnDown()) {
      delay(50);
      if (btnDown()) {
        // OK button is definitely pushed so now detect how long it is pressed.
        i = 0;
        while ( btnDown() ) {
          delay(10);
          i++;
          if (i > BUTTON_DELAY_2 )printLine2((char *)"Exit Step 1");
        }
        if (i <= BUTTON_DELAY_2 ) {
          Cal_Freq_Index = Cal_Freq_Index - 1;
          if ( Cal_Freq_Index < 0 )Cal_Freq_Index = 6;
          Update_Freq();
        } else {
          printLine(1, (char*)"");
          while (btnDown())delay(50);
          delay(100);
          printLine1((char *)"Step 2: Best Sound");
          printLine2((char *)"Encoder to Next Step");
          menuSetupCalibration();
          menuSetupCarrier();
          Blank_Screen();
          return;
        }
      }
    }
    CaldoTuning();
    delay(100);
  }

}



void Menu_Backup_EEPROM(int btn) {
  int i;

  if (!btn) {
    printLine1((char*)"BACKUP EEPROM DATA");
    return;
  }
  printLine2((char*)"Press Choose exit");
  printLine3((char *)"Writing to SD");
  while (Menu_BTN_Down()) delay(50);
  if (!Backup_EEPROM_to_SD() ) {
    printLine4((char *)"BACKUP - FAILED");
  } else {
    for (i = 0; i < 10; i++) {
      Put_SDEEPROM( 100 + (i * 4), TableFREQS[0][i] );
      Put_SDEEPROM( 140 + (i * 4), TableMODES[0][i] );
    }
    for (i = 0; i < 10; i++) {
      Put_SDEEPROM( 200 + (i * 4), TableFREQS[1][i] );
      Put_SDEEPROM( 240 + (i * 4), TableMODES[1][i] );
    }
    for (i = 0; i < 10; i++) {
      Put_SDEEPROM( 300 + (i * 4), TableFREQS[2][i] );
      Put_SDEEPROM( 340 + (i * 4), TableMODES[2][i] );
    }
    for (i = 0; i < 10; i++) {
      Put_SDEEPROM( 400 + (i * 4), TableFREQS[3][i] );
      Put_SDEEPROM( 440 + (i * 4), TableMODES[3][i] );
    }
    printLine4((char *)"Successful");
  }
  while (!Menu_BTN_Down()) delay(50);
  printLine2((char *)"   ");
  printLine3((char *)"   ");
  printLine4((char *)"   ");
  while (Menu_BTN_Down()) delay(50);
}

void Menu_Activate_CW_Filter( int btn ) {
  int s;

  CW_Filter_Present = Get_SDEEPROM( CW_FILTER_PRESENT );
  if ( (CW_Filter_Present < 0) || (CW_Filter_Present > 1)) CW_Filter_Present = 0;
  if (!btn) {
    if ( CW_Filter_Present )printLine1((char *)"CW Filter IN " );
    else printLine1((char *)"CW Filter OUT");
    return;
  }
  printLine2((char*)"Press Choose to set");
  lcd.setCursor(16, 0);
  while (Menu_BTN_Down())delay(10);
  while ( !Menu_BTN_Down() ) {
    delay(50);
    s = Menu_Select_Encoder_Read();
    if (s != 0 ) {
      if ( s < -1 ) {
        CW_Filter_Present = false;
        CW_Filter_Offset = 0;
        printLine1((char*)"CW Filter OUT");
      } else if ( s > 1 ) {
        CW_Filter_Present = true;
        printLine1((char*)"CW Filter IN ");
      }
      lcd.setCursor(16, 0);
      //   delay(1000);
    }
  }
  Put_SDEEPROM( CW_FILTER_PRESENT, CW_Filter_Present );
  Put_SDEEPROM( CW_FILTER_CALIBRATED, false );
  CW_Calibrated  = false;
  if ( CW_Filter_Present ) {
    printLine2((char*)"MUST CALIBRATE");
    delay(3000);
  }
  setFrequency(frequency);
  printLine2((char*)"   ");
  while ( Menu_BTN_Down())delay(10);


}

void Menu_Activate_Smeter( int btn ) {
  int s;

  Smeter_Active = Get_SDEEPROM( SMETER_ACTIVE );
  if ( (Smeter_Active < 0) || (Smeter_Active > 1)) Smeter_Active = 0;
  if (!btn) {
    if ( Smeter_Active )printLine1((char *)"S-meter Display ON" );
    else printLine1((char *)"S-meter Display OFF");
    return;
  }
  printLine2((char*)"Press Choose to set");
  lcd.setCursor(16, 0);
  while (Menu_BTN_Down())delay(10);
  while ( !Menu_BTN_Down() ) {
    delay(50);
    s = Menu_Select_Encoder_Read();
    if (s != 0 ) {
      if ( s < -1 ) {
        Smeter_Active = false;
        printLine1((char*)"S-meter Display OFF");
      } else if ( s > 1 ) {
        Smeter_Active = true;
        printLine1((char*)"S-meter Display ON ");
      }
      lcd.setCursor(16, 0);
      //   delay(1000);
    }
  }
  Put_SDEEPROM( SMETER_ACTIVE, Smeter_Active );
  printLine2((char*)"   ");
  while ( Menu_BTN_Down())delay(10);


}




void Menu_Activate_Volts( int btn ) {
  int s;

  Volts_Active = Get_SDEEPROM( VOLTS );
  if ( (Volts_Active < 0) || (Volts_Active > 1)) Volts_Active = 0;
  if (!btn) {
    if ( Volts_Active )printLine1((char *)"Volts Display ON " );
    else printLine1((char *)"Volts Display OFF");
    return;
  }
  printLine2((char*)"Press Choose to set");
  lcd.setCursor(14, 0);
  while (Menu_BTN_Down())delay(10);
  while (!Enter_Exit_Down() &&  !Menu_BTN_Down() ) {
    delay(50);
    s = Menu_Select_Encoder_Read();
    if (s != 0 ) {
      if ( s < -1 ) {
        Volts_Active = 0;
        printLine1((char*)"Volts Display OFF");
      } else if ( s > +1 ) {
        Volts_Active = 1;
        printLine1((char*)"Volts Display ON ");
      }
      lcd.setCursor(14, 0);
    }
  }
  if ( Menu_BTN_Down() ) {
    Put_SDEEPROM( VOLTS, Volts_Active );
  } else {
    Volts_Active = Get_SDEEPROM( VOLTS );
    if ( (Volts_Active < 0) || (Volts_Active > 1)) Volts_Active = 0;
    if ( Volts_Active )printLine1((char *)"Volts Display ON " );
    else printLine1((char *)"Volts Display OFF");
  }
  printLine2((char*)" ");
  while ( Menu_BTN_Down())delay(10);


}


void Menu_Setup_Volts( int btn ) {
  char Voltage_Text[21];
  char c[21];
  int s, i = 1;

  if (!btn) {
    printLine1((char *)"CALIBRATE VOLTMETER");
    return;
  }
  Volt_Divider = Get_SDEEPROM( VOLT_DIVIDE );
  if (  (Volt_Divider > 2000) || (Volt_Divider < 0))Volt_Divider = 982;
  printLine1((char *)"Match Your VCC");
  printLine2((char *)"Press Choose to set");
  while (Menu_BTN_Down())delay(10);
  while (!Menu_BTN_Down()) {
    delay(50);
    s = Menu_Select_Encoder_Read();
    if (s > 4)
      Volt_Divider--;
    else if (s >  2)
      Volt_Divider--;
    else if (s >  0)
      Volt_Divider--;
    else if (s <  0)
      Volt_Divider++;
    else if (s < -2)
      Volt_Divider++;
    else if (s < -4)
      Volt_Divider++;
    else if ( i == 1)i = 0;
    else continue;
    itoa(Volt_Divider, Voltage_Text, 10);
    printLine3(Voltage_Text);
    Ubitx_Voltage = analogRead(UBIT_VCC);
    Converted_Ubitx_Voltage = (Ubitx_Voltage * (unsigned long)1500);
    Ubitx_Voltage = analogRead(UBIT_VCC);
    temp_value = Ubitx_Voltage;
    temp_value = (temp_value * 15.0);
    temp_value = temp_value  / (float)Volt_Divider;
    sprintf(c, "%02.1fV", temp_value);
    printLine(3, c);
  }
  printLine1((char *)"");
  printLine2((char *)"");
  printLine3((char *)"");
  printLine4((char *)"");
  Put_SDEEPROM( VOLT_DIVIDE, Volt_Divider );
  while (Menu_BTN_Down())delay(10);
}


void Menu_Select_Baud( int btn ) {
  int i;
  int BaudmenuOn;


  if (!btn) {
    printLine1((char *)"CAT Baud Rate");
    return;
  }
  printLine2((char *)"Press Choose to set");
  ubitx_baud = Last_Baud;
  printLine3(ubitx_CAT_Baud_rates[ubitx_baud]);
  lcd.setCursor(0, 2 );
  while (Menu_BTN_Down())delay(10);
  while ( !Menu_BTN_Down() ) {
    delay(50);
    i = Menu_Select_Encoder_Read();
    if ( i != 0) {
      if (i > 1) {
        ubitx_baud++;
        if ( ubitx_baud > 7) ubitx_baud = 7;
      } else if (i < -1 ) {
        ubitx_baud--;
        if ( ubitx_baud < 0) ubitx_baud = 0;
      }
      printLine3(ubitx_CAT_Baud_rates[ubitx_baud]);
      lcd.setCursor(0, 2 );
      //   delay(500);
    }
  }
  Last_Baud = ubitx_baud;
  BaudmenuOn = ubitx_baud;
  Put_SDEEPROM( BAUD, BaudmenuOn );
  printLine4((char *)"must reboot");
  lcd.setCursor(0, 4 );
  delay(2000);
  printLine2((char *)"   ");
  printLine3((char *)"   ");
  printLine4((char *)"   ");

}

int get_number(int min, int max , int start_value) {
  int answer;
  int i;
  char numbr[5];

  answer = start_value;
  sprintf(numbr, "%02d", answer);
  printLineXI(4, 0, numbr);
  lcd.setCursor(0, 4 );
  while ( Menu_BTN_Down() ) delay(10);
  while ( !Menu_BTN_Down() ) {
    delay(50);
    if ( Enter_Exit_Down() ) {
      printLine4((char*)"     ");
      return (-1);
    }
    i = Menu_Select_Encoder_Read();
    if ( i != 0) {
      if (i > 1) {
        answer++;
        if ( answer > max) answer = max;
      } else if (i < -1 ) {
        answer--;
        if ( answer < min ) answer = min;
      }
      sprintf(numbr, "%02d", answer);
      printLine4(numbr);
      lcd.setCursor(0, 4 );
    }
  }
  while ( Menu_BTN_Down() ) delay(10);
  return ( answer);
}

int Time_Month = 5, Time_Day = 20, Time_Year = 2018, Time_Hour = 13, Time_Minute = 14;




void Menu_Set_Time( int btn ) {
  char Time_Now[17];
  int s;
  bool Current_Answer;

  if (!btn) {
    printLine1((char *)"SET CLOCK/TIME");
    return;
  }
  while (Menu_BTN_Down())delay(10);
  printLine2((char *)"Choose-> ON/OFF");
  if ( Clock_Active )printLine1((char *)"To Turn Clock ON ");
  else printLine1((char *)"To Turn Clock OFF");
  ////////////////////////////////////////////////////////////////
  lcd.setCursor(14, 0);
  while (Menu_BTN_Down())delay(10);
  while (!Enter_Exit_Down() &&  !Menu_BTN_Down() ) {
    delay(50);
    s = Menu_Select_Encoder_Read();
    if (s != 0 ) {
      if ( s > +1 ) {
        Clock_Active = false;
        printLine1((char*)"To Turn Clock OFF");
      } else if ( s < -1 ) {
        Clock_Active = true;
        printLine1((char*)"To Turn Clock ON ");
      }
      lcd.setCursor(14, 0);
    }
  }
  if ( Menu_BTN_Down()) {
    Put_SDEEPROM( CLOCK_ACTIVE, Clock_Active );
  }
  if ( Menu_BTN_Down() && Clock_Active ) {
    printLine2((char *)"Choose-> YES/NO");
    printLine1((char *)"Set date/Time YES");
    ////////////////////////////////////////////////////////////////
    lcd.setCursor(14, 0);
    while (Menu_BTN_Down())delay(10);
    Current_Answer = 1;
    while (!Enter_Exit_Down() &&  !Menu_BTN_Down() ) {
      delay(50);
      s = Menu_Select_Encoder_Read();
      if (s != 0 ) {
        if ( s > +1 ) {
          Current_Answer = 0;
          printLine1((char*)"Set date/Time NO ");
        } else if ( s < -1 ) {
          Current_Answer = 1;
          printLine1((char*)"Set date/Time YES");
        }
        lcd.setCursor(14, 0);
      }
    }
    if ( Menu_BTN_Down() && (Current_Answer == 1)) {



      printLine1((char *)"Choose-> Next");
      sprintf(Time_Now, "%02d/%02d/%4d %02d:%02d", Time_Month, Time_Day, Time_Year, Time_Hour, Time_Minute);
      printLine2(Time_Now);
      printLine3((char*)"Select Month");
      printLine4((char*)"      ");
      if ( (Time_Month = get_number(0, 12, Time_Month)) < 0 )return;
      //-------------------------------------------
      sprintf(Time_Now, "%02d/%02d/%4d %02d:%02d", Time_Month, Time_Day, Time_Year, Time_Hour, Time_Minute);
      printLine2(Time_Now);
      printLine3((char*)"Select Day");
      printLine4((char*)"   ");
      if ( (Time_Day = get_number(0, 31, Time_Day)) < 0 )return;
      //-------------------------------------------
      sprintf(Time_Now, "%02d/%02d/%4d %02d:%02d", Time_Month, Time_Day, Time_Year, Time_Hour, Time_Minute);
      printLine2(Time_Now);
      printLine3((char*)"Select Year");
      printLine4((char*)"   ");
      if ( (Time_Year = get_number(0, 9999, Time_Year)) < 0 )return;
      //-------------------------------------------
      sprintf(Time_Now, "%02d/%02d/%4d %02d:%02d", Time_Month, Time_Day, Time_Year, Time_Hour, Time_Minute);
      printLine2(Time_Now);
      printLine3((char*)"Select Hour");
      printLine4((char*)"     ");
      if ( (Time_Hour = get_number(0, 23, Time_Hour)) < 0)return;
      //-------------------------------------------
      sprintf(Time_Now, "%02d/%02d/%4d %02d:%02d", Time_Month, Time_Day, Time_Year, Time_Hour, Time_Minute);
      printLine2(Time_Now);
      printLine3((char*)"Select Minute");
      printLine4((char*)"   ");
      if ( (Time_Minute = get_number(0, 59, Time_Minute)) < 0 )return;
      sprintf(Time_Now, "%02d/%02d/%4d %02d:%02d", Time_Month, Time_Day, Time_Year, Time_Hour, Time_Minute);
      printLine2(Time_Now);
      printLine1((char *)"Choose-> Set Time");
      printLine3((char*)"     ");
      printLine4((char*)"     ");

      while (!Menu_BTN_Down() && !Enter_Exit_Down())delay(10);
      if (Enter_Exit_Down()) {
        while (Enter_Exit_Down())delay(10);
      } else {
        while ( Menu_BTN_Down())delay(10);
        //setTime(hours, minutes, seconds, days, months, years);
        setTime(Time_Hour, Time_Minute, 0, Time_Day, Time_Month, Time_Year);
        Teensy3Clock.set(now());
      }
    }
  }
  Config_Loop = 0;


}

void Do_Configuration(void) {
  int Config_Select = 0;
  int s;
  bool btnState;

  //wait for the button to be raised up
  while (Menu_BTN_Down())
    delay(50);
  delay(50);  //debounce


  Blank_Screen();
  Config_Loop = 2;

  while (!Enter_Exit_Down() && Config_Loop ) {

    btnState = Menu_BTN_Down();
    if (Config_Select < 10)
      Menu_Set_Pf(btnState);
    else if (Config_Select < 20)
      Menu_Calibrate(btnState);
    else if (Config_Select < 30)
      Menu_Activate_Volts(btnState);
    else if (Config_Select < 40)
      Menu_Setup_Volts(btnState);
    else if (Config_Select < 50)
      Menu_Activate_Smeter(btnState);
    else if (Config_Select < 60)
      Menu_Activate_CW_Filter(btnState);
    else if (Config_Select < 70)
      Menu_Select_Baud(btnState);
    else if (Config_Select < 80)
      Menu_Backup_EEPROM(btnState);
    else if (Config_Select < 90)
      Menu_Set_Time(btnState);


    s = Menu_Select_Encoder_Read();
    Config_Select += s;
    if ( Config_Select > 89 )Config_Select = 0;
    if ( Config_Select < 0 )Config_Select = 89;
  }
  Blank_Screen();
  //debounce the button
  while (Enter_Exit_Down())
    delay(50);
  delay(50);
  updateDisplay();
}



