/** Menus
    File Name ubitx_menu.cpp
    The Radio menus are accessed by tapping on the function button.
    - The main loop() constantly looks for a button press and calls doMenu() when it detects
    a function button press.
    - As the encoder is rotated, at every 10th pulse, the next or the previous menu
    item is displayed. Each menu item is controlled by it's own function.
    - Each menu function may be called to display itself
    - Each of these menu routines is called with a button parameter.
    - The btn flag denotes if the menu item was clicked on or not.
    - If the menu item is clicked on, then it is selected,
    - If the menu item is NOT clicked on, then the menu's prompt is to be displayed
*/
#include <Arduino.h>
#include <EEPROM.h>
#include "ubitx.h"

#include <LiquidCrystal_Wire2_I2C.h>
extern LiquidCrystal_Wire2_I2C lcd;//  indicate that "lcd" was already defined in <ubitx_V1_08_I2C>

extern void stopTx(void);
extern void printLine2(char *c);
extern void printLine3(char *c);
extern void printLine4(char *c);
extern bool btnDown(void);
extern void ritDisable(void);
extern void ritEnable(void);
extern int  enc_read(void);
extern void setFrequency(unsigned long f);
extern void updateDisplay(void);
extern void printLine1(char *c);
extern void startTx(byte txMode);
extern void si5351bx_setfreq(uint8_t clknum, uint32_t fout);
extern void cwKeydown(void);
extern void cwKeyUp(void);
extern int Test_Button( int button, char * title1, char * title2, char * title3, char * title4 );
extern void printLineXI(char linenmbr, char index, char *c);
extern int Menu_Select_Encoder_Read(void);
extern bool Choose_Down(void);
extern bool Enter_Exit_Down(void);
extern bool Cat_Lock;
extern bool Put_SDEEPROM( int index, long Value );
extern long Get_SDEEPROM( int index );
extern volatile int LastSmeter;
extern void Display_Mem_Selection( void );
extern void sendKey(char c);
extern volatile boolean Key_Send_Buffer_Full;
extern volatile int     Key_Send_Buffer_IN_Index;
extern volatile int     Key_Send_Buffer_OUT_Index;


extern unsigned long frequency;
extern char ritOn;
extern char splitOn;
extern byte menuOn;
extern char vfoActive;
extern unsigned long vfoA;
extern unsigned long vfoB;
extern boolean modeCalibrate;
extern int cwSpeed;
extern char c[21], b[21];
extern char keyDown;
extern unsigned long cwTimeout;
extern unsigned long sideTone;
extern char Version_Text[16];
extern char Version_Date[16];
extern int Menu_Select_Index[7][2];         // new menu
extern int Menu_Index;                      // new menu
extern volatile int ubitx_mode;
extern int Vfo_Mode[2];
extern unsigned volatile char IAMBIC;  // 0 for Iambic A, 0x10 for Iambic B
extern unsigned volatile char PDLSWAP;     // 0x00 for normal, 0x08 for swap
extern volatile unsigned char keyerControl;
extern unsigned volatile char IAMBIC;
extern bool Volts_Active;
extern volatile bool Turn_Off_Carrier_in_Progress;
extern volatile unsigned long  Turn_Off_Carrier_Timer_Count;
extern bool Tune_Lock;
extern volatile int     Key_Send_Buffer_Count;

extern char Keyboard_Input_Buffer[40];
int KeyIndex = 0;
int OutKeyIndex = 0;
int SendKeyIndex = 0;
int Keyboard_Value;

bool Key_Displayed = false;
bool Creating_Buffered_Text = false;
int Keyboard_Mode = 0;

//extern int Memory_Number;
bool Memories_Active = false;

extern char ubitx_mode_text[7][5];

int wpm;
extern int Mem_Number;
int Memory_Table_Select = 0;

int Menu_Cursor_Line_Postions[9] =  {1, 1, 1, 1, 1};
int Menu_Cursor_Index_Postions[9] = {0, 5, 6, 8, 13};
int Max_Menus_Items = 4;
int Current_Menu = 0;
int Last_Menu = 0;
int Last_Mode = 0;

////////////////////////////////////////////////////////////
extern int   Working_Copy_Modes[10];
extern unsigned long Working_Copy_Frequencies[10];

int TableMODES[4][10] = { 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
                          1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
                          0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                          1, 1, 2, 1, 2, 1, 1, 1, 1, 1
                        };


unsigned long TableFREQS[4][10] = { 3560000, 7030000, 7122000, 10106000, 10116000, 14060000, 18096000, 21060000, 24906000, 28060000,
                                    3985000, 7285000, 7090000, 14285000, 18130000, 21285000, 21385000, 24950000, 28365000, 28385000,
                                    3580000, 3576000, 7070000, 7076000, 3562000, 7277500, 7280000, 7280000, 7280000, 7280000,
                                    7277000, 3920000, 3984000, 3547000, 7290000, 3562000, 7277500, 7280000, 7280000, 7280000
                                  };




char Key_buffer0[2];
char Key_buffer1[21];
char Key_buffer2[21];
unsigned long Key_Freq;

////////////////////////////////////////////////////////////


void Print_Menu_Line(void) {
  char Menu_Text[21];


  if ( ubitx_mode == MODE_USB  )strcpy(Menu_Text, "USB");
  else if ( ubitx_mode == MODE_LSB   )strcpy(Menu_Text, "LSB");
  else if ( ubitx_mode == MODE_CW    )strcpy(Menu_Text, "CW ");
  else if ( ubitx_mode == MODE_CWR   )strcpy(Menu_Text, "CWr");
  else if ( ubitx_mode == MODE_SWU   )strcpy(Menu_Text, "SWU");
  else if ( ubitx_mode == MODE_SWL   )strcpy(Menu_Text, "SWL");
  else if ( ubitx_mode == MODE_CWP   )strcpy(Menu_Text, "CWP");
  if ( PDLSWAP == 0 )strcat(Menu_Text, "  N");
  else strcat(Menu_Text, "  R");
  if ( IAMBIC == 0 )strcat(Menu_Text, "A T");
  else strcat(Menu_Text, "B T");
  itoa(sideTone, b, 10);
  strcat(Menu_Text, b);
  strcat(Menu_Text, " S");
  itoa(wpm, b, 10);
  strcat(Menu_Text, b);
  printLine2(Menu_Text);


  lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu], Menu_Cursor_Line_Postions[Current_Menu] );

}

void SetIambicA( void ) {
  int temp;

  IAMBIC = 0;
  keyerControl = keyerControl & 0xEF;
  temp = IAMBIC;
  Put_SDEEPROM( IAMBIC_MODE, temp );
  delay(200);
}

void SetIambicB( void ) {
  int temp;

  IAMBIC = 0x10;
  keyerControl = keyerControl | IAMBIC;
  temp = IAMBIC;
  Put_SDEEPROM( IAMBIC_MODE, temp );
  delay(200);
}

void menuIambicToggle( void ) {

  if ( IAMBIC == 0 )SetIambicB();
  else SetIambicA();
  menuOn = 0;

}

void SetPaddleSwapN( void ) {

  PDLSWAP = 0;
  keyerControl = keyerControl & 0xF7;

}

void SetPaddleSwapR( void ) {

  PDLSWAP = 0x08;
  keyerControl = keyerControl | PDLSWAP;

}



void menuPaddleSwapToggle( void ) {
  int temp;

  if ( PDLSWAP == 0 ) SetPaddleSwapR();
  else  SetPaddleSwapN();
  temp = PDLSWAP;
  Put_SDEEPROM( PDL_SWAP_VAL, temp );
  delay(200);
  menuOn = 0;


}


void menuVfoToggle( void ) {

  if (splitOn) {
    if (vfoActive == VFO_B)vfoActive = VFO_A;
    else vfoActive = VFO_B;
  } else {
    if (vfoActive == VFO_B) {
      vfoB = frequency;
      Put_SDEEPROM( VFO_B, frequency );
      vfoActive = VFO_A;
      frequency = vfoA;
    } else {
      vfoA = frequency;
      Put_SDEEPROM( VFO_A, frequency );
      vfoActive = VFO_B;
      frequency = vfoB;
    }
  }
  ritDisable();
  setFrequency(frequency);
  updateDisplay();

}

void menuRitToggle( void ) {
  if (ritOn == 0) {
    ritEnable();
  } else {
    ritDisable();
  }
  menuOn = 0;
  delay(100);
  updateDisplay();
}

void menuModeSelect( void ) {
  int modemenuOn = 2;
  int i;
  int temp;

  temp = Last_Mode;
  //  printLine2(ubitx_mode_text[ubitx_mode]);
  lcd.setCursor(0, 1 );
  //wait for the button menu select button to be lifted)
  while (btnDown()) delay(50);
  delay(50);
  ritDisable();


  while (modemenuOn) {
    i = Menu_Select_Encoder_Read();
    if ( i != 0) {
      if (i > 1) {
        temp++;
        if ( temp > 6) temp = 6;
      } else if (i < -1 ) {
        temp--;
        if ( temp < 0) temp = 0;
      }
      printLineXI(1, 0, ubitx_mode_text[temp]);
      lcd.setCursor(0, 1 );
      delay(500);
    }
    if ( Enter_Exit_Down() ) {
      while ( btnDown() ) delay(50);
      modemenuOn = 0;
    }
  }
  Last_Mode = temp;
  ubitx_mode = Last_Mode;
  menuOn = ubitx_mode;
  Put_SDEEPROM( MODE, menuOn );
  delay(200);
  setFrequency(frequency);
  updateDisplay();
  menuOn = 0;
}



void menuSplitToggle( void ) {

  if ( splitOn ) {
    splitOn = 0;
  } else {
    splitOn = 1;
    vfoB = vfoA;
    ritDisable();
  }
  updateDisplay();
}




void menuCWSpeed( void ) {
  int knob = 0;

  lcd.setCursor(15, 1 );
  while (!Enter_Exit_Down() && digitalRead(PTT) == HIGH) {

    knob = Menu_Select_Encoder_Read();
    if (knob != 0) {
      if (wpm > 5 && knob < 0)
        wpm--;
      if (wpm < 50 && knob > 0)
        wpm++;
      itoa(wpm, c, 10);
      printLineXI(1, 14, c);
      lcd.setCursor(15, 1 );
    }
    //abort if this button is down
    if (btnDown())
      //re-enable the clock1 and clock 2
      break;
  }

  //save the setting
  cwSpeed = 600 / wpm;
  Put_SDEEPROM( CW_SPEED, wpm );
  delay(200);
  menuOn = 0;
}



void menuSetupCwTone( void ) {
  int knob = 0;


  delay(200);
  tone(CW_TONE, sideTone);
  lcd.setCursor(11, 1 );

  //disable all clock 1 and clock 2
  while (digitalRead(PTT) == LOW || !Enter_Exit_Down())
  {
    knob = Menu_Select_Encoder_Read();

    if (knob > 0 && sideTone < 990)
      sideTone += 5;
    else if (knob < 0 && sideTone > 100 )
      sideTone -= 5;
    else
      continue; //don't update the frequency or the display

    tone(CW_TONE, sideTone);
    itoa(sideTone, b, 10);
    printLineXI(1, 9, b);
    lcd.setCursor(11, 1 );

    delay(100);
  }
  noTone(CW_TONE);
  //save the setting
  Put_SDEEPROM( CW_SIDETONE, sideTone );
  delay(200);
  menuOn = 0;
}

void Process_Volts( void ) {

  if (Volts_Active) {
    Volts_Active = false;
    printLineXI(0, 0, (char *)"     ");
  } else {
    Volts_Active = true;
  }
  menuOn = Volts_Active;
  Put_SDEEPROM( VOLTS, menuOn );
  delay(200);
  menuOn = 0;


}


void Exit_Keystroke( void ) {
  printLine2((char *)" ");
  printLine3((char *)" ");
  //printLineXI(2,0,(char *)"  ");
  printLineXI(2, 1, (char *)" ");
  updateDisplay();
  LastSmeter = 0;
  Display_Mem_Selection();

}

void Process_Idle( void ) {

  switch ( Keyboard_Input_Buffer[OutKeyIndex] ) {

    case CTL_c:
      Keyboard_Mode = KEY_CW;
      strcpy(Key_buffer2, ">                   ");
      printLineXI(0, 0, Key_buffer2);
      strcpy(Key_buffer1, "");
      strcpy(Key_buffer2, "");
      Key_Displayed = true;
      Key_Freq = 0;
      break;

    case CTL_f:
      Keyboard_Mode = KEY_FREQ;
      strcpy(Key_buffer2, "F=        ");
      printLineXI(1, 0, Key_buffer2);
      strcpy(Key_buffer1, "");
      strcpy(Key_buffer2, "");
      Key_Displayed = true;
      Key_Freq = 0;
      break;

    case CTL_m:
      ritDisable();
      ubitx_mode++;
      if ( ubitx_mode > 6) ubitx_mode = 0;
      Put_SDEEPROM( MODE, ubitx_mode );
      delay(200);
      setFrequency(frequency);
      updateDisplay();
      break;

    case CTL_i:
      if ( IAMBIC == 0 ) {
        SetIambicB();
      } else {
        SetIambicA();
      }
      Print_Menu_Line();
      delay(2000);
      Exit_Keystroke();
      break;

    case CTL_p:
      if ( PDLSWAP == 0 ) {
        SetPaddleSwapR();
      } else {
        SetPaddleSwapN();
      }
      Put_SDEEPROM( PDL_SWAP_VAL, PDLSWAP );
      Print_Menu_Line();
      delay(2000);
      Exit_Keystroke();
      break;

    case CTL_v:
      menuVfoToggle();
      break;

    case CTL_r:
      if ( splitOn == 0 ) {
        if (ritOn) ritDisable();
        else ritEnable();
      }
      break;

    case CTL_s:
      if ( !ritOn) menuSplitToggle();
      break;

    case CTL_w:
      Keyboard_Mode = KEY_WPM;
      strcpy(Key_buffer2, "          ");
      printLineXI(1, 0, Key_buffer2);
      sprintf(Key_buffer2, "w=%d", wpm);
      printLineXI(1, 0, Key_buffer2);
      strcpy(Key_buffer1, "");
      strcpy(Key_buffer2, "");
      Key_Displayed = true;
      Key_Freq = wpm;
      break;

    case CTL_t:
      Keyboard_Mode = KEY_TONE;
      strcpy(Key_buffer2, "          ");
      printLineXI(1, 0, Key_buffer2);
      sprintf(Key_buffer2, "t=%d", (int)sideTone);
      printLineXI(1, 0, Key_buffer2);
      strcpy(Key_buffer1, "");
      strcpy(Key_buffer2, "");
      Key_Displayed = true;
      Key_Freq = sideTone;
      break;

    case CTL_IF:
      printLineXI(2, 0, (char*)">");
      delay(1000);
      printLineXI(2, 0, (char*)" ");
      break;

    default:
     // sprintf(Key_buffer2, "key=%x", Keyboard_Input_Buffer[OutKeyIndex]);
      //printLineXI(1, 0, Key_buffer2);
      break;
  }
  if ( Tune_Lock )printLineXI(2, 0, (char*)"LOCKON ");
  else printLineXI(2, 0, (char*)"      ");
  KeyIndex = 0;
  SendKeyIndex = 0;
}


void Process_Speed( void ) {

  KeyIndex = 0;
  Key_Displayed = false;
  Keyboard_Value = Keyboard_Input_Buffer[0];
  if ( Keyboard_Value == 0x0a ) {
    if ( Key_Freq >= 5 &&  Key_Freq <= 50 ) {
      wpm = Key_Freq;
      cwSpeed = 600 / wpm;
      Put_SDEEPROM( CW_SPEED, wpm );
    } else {
      printLineXI(1, 0, (char*)"Bad input");
      delay(500);
    }
    printLineXI(1, 0, (char*)"          ");
    Keyboard_Mode = KEY_IDLE;
    Exit_Keystroke();
  } else if ( Keyboard_Value == 0x1b ) {
    printLineXI(1, 0, (char*)"          ");
    Keyboard_Mode = KEY_IDLE;
    updateDisplay();
  } else if ( (Keyboard_Value == 0x08) || (Keyboard_Value == 0x7F) ) {
    if ( SendKeyIndex > 0 ) {
      SendKeyIndex = SendKeyIndex - 1;
      Key_buffer2[strlen(Key_buffer2) - 1] = 0;
      Key_Freq = atol(Key_buffer2);
      if ( !Key_Displayed ) {
        printLineXI(1, 2, (char*)"        ");
        printLineXI(1, 2, Key_buffer2);
      }
      Key_Displayed = true;
    }
  } else {
    SendKeyIndex = SendKeyIndex + 1;
    if ( SendKeyIndex > 19 ) {
      printLineXI(0, 0, (char*)">                   ");
      SendKeyIndex = 1;
      strcpy(Key_buffer1, "");
    }
    sprintf(Key_buffer1, "%c", Keyboard_Value);
    strcat(Key_buffer2, Key_buffer1);
    Key_Freq = atol(Key_buffer2);
    if ( !Key_Displayed ) {
      printLineXI(1, 2, (char*)"        ");
      printLineXI(1, 2, Key_buffer2);
    }
    Key_Displayed = true;
  }
}


void Process_Tone( void ) {

  KeyIndex = 0;
  Key_Displayed = false;
  Keyboard_Value = Keyboard_Input_Buffer[0];
  if ( Keyboard_Value == 0x0a ) {
    if ( Key_Freq >= 100 &&  Key_Freq <= 990 ) {
      sideTone = Key_Freq;
      Put_SDEEPROM( CW_SIDETONE, sideTone );
    } else {
      printLineXI(1, 0, (char*)"Bad input");
      delay(500);
    }
    printLineXI(1, 0, (char*)"          ");
    Keyboard_Mode = KEY_IDLE;
    Exit_Keystroke();
  } else if ( Keyboard_Value == 0x1b ) {
    printLineXI(1, 0, (char*)"          ");
    Keyboard_Mode = KEY_IDLE;
    Exit_Keystroke();
  } else if ( (Keyboard_Value == 0x08) || (Keyboard_Value == 0x7F) ) {
    if ( SendKeyIndex > 0 ) {
      SendKeyIndex = SendKeyIndex - 1;
      Key_buffer2[strlen(Key_buffer2) - 1] = 0;
      Key_Freq = atol(Key_buffer2);
      if ( !Key_Displayed ) {
        printLineXI(1, 2, (char*)"        ");
        printLineXI(1, 2, Key_buffer2);
      }
      Key_Displayed = true;
    }
  } else {
    SendKeyIndex = SendKeyIndex + 1;
    sprintf(Key_buffer1, "%c", Keyboard_Value);
    strcat(Key_buffer2, Key_buffer1);
    Key_Freq = atol(Key_buffer2);
    if ( !Key_Displayed ) {
      printLineXI(1, 2, (char*)"        ");
      printLineXI(1, 2, Key_buffer2);
    }
    Key_Displayed = true;
  }
}




void Process_CW( void ) {
  int i;

  KeyIndex = 0;
  Key_Displayed = false;
  Keyboard_Value = Keyboard_Input_Buffer[0];
  if ( Keyboard_Value == 0x1b ) {
    noInterrupts();
    Key_Send_Buffer_Count = 0;
    Key_Send_Buffer_IN_Index = 0;
    Key_Send_Buffer_OUT_Index = 0;
    sendKey(toupper(0));
    interrupts();
    printLineXI(0, 0, (char*)"                    ");
    Exit_Keystroke();
    Keyboard_Mode = KEY_IDLE;
  } else if ( Keyboard_Value == 0x0A ) {
    sendKey(toupper(0));
    printLineXI(0, 0, (char*)"                    ");
    Exit_Keystroke();
    Keyboard_Mode = KEY_IDLE;
    /*
      } else if ( (Keyboard_Value == 0x08) || (Keyboard_Value == 0x7F) ) {
        if ( SendKeyIndex > 1 ) {
          if ( SendKeyIndex > 19 ) {
            SendKeyIndex = SendKeyIndex - 1;
            Key_buffer2[strlen(Key_buffer2) - 1] = 0;
            if ( !Key_Displayed ) {
              printLineXI(1, 0, (char*)"                    ");
              printLineXI(1, 0, Key_buffer2);
            }
            Key_Displayed = true;
          } else {
            SendKeyIndex = SendKeyIndex - 1;
            Key_buffer1[strlen(Key_buffer1) - 1] = 0;
            if ( !Key_Displayed ) {
              printLineXI(0, 1, (char*)"                   ");
              printLineXI(0, 1, Key_buffer1);
            }
            Key_Displayed = true;
          }
        }
    */
  } else {
    if ( !Key_Send_Buffer_Full ) {
      SendKeyIndex = SendKeyIndex + 1;
      sprintf(Key_buffer0, "%c", toupper(Keyboard_Value));
      if ( SendKeyIndex > 19  ) {
        for (i = 0; i < 18; i++)Key_buffer1[i] = Key_buffer1[i + 1];
        SendKeyIndex = 19;
        Key_buffer1[18] = Key_buffer0[0];
        Key_buffer1[19] = 0;
      } else strcat(Key_buffer1, Key_buffer0);
      printLineXI(0, 1, Key_buffer1);
      if(Keyboard_Value == ' ')Keyboard_Value = 1;
      sendKey(toupper(Keyboard_Value));
    }
  }
}


void Process_Freq( void ) {

  KeyIndex = 0;
  Key_Displayed = false;
  Keyboard_Value = Keyboard_Input_Buffer[0];

  if ( Keyboard_Value == 0x0a ) {
    if ( Key_Freq >= 400000 && Key_Freq <= 30000000 ) {
      frequency = Key_Freq;
      if ( vfoActive == VFO_A ) vfoA = frequency;
      else vfoB = frequency;
      setFrequency(frequency);
    } else {
      printLineXI(1, 0, (char*)"Bad input  ");
      delay(500);
    }
    printLineXI(1, 0, (char*)"          ");
    Keyboard_Mode = KEY_IDLE;
    Exit_Keystroke();
  } else if ( Keyboard_Value == 0x1b ) {
    printLineXI(1, 0, (char*)"          ");
    Keyboard_Mode = KEY_IDLE;
    Exit_Keystroke();
  } else if ( (Keyboard_Value == 0x08) || (Keyboard_Value == 0x7F) ) {
    if ( SendKeyIndex > 0 ) {
      SendKeyIndex = SendKeyIndex - 1;
      Key_buffer2[strlen(Key_buffer2) - 1] = 0;
      Key_Freq = atol(Key_buffer2);
      if ( !Key_Displayed ) {
        printLineXI(1, 2, (char*)"        ");
        printLineXI(1, 2, Key_buffer2);
      }
      Key_Displayed = true;
    }
  } else {
    SendKeyIndex = SendKeyIndex + 1;
    sprintf(Key_buffer1, "%c", Keyboard_Value );
    strcat(Key_buffer2, Key_buffer1);
    Key_Freq = atol(Key_buffer2);
    if ( !Key_Displayed )printLineXI(1, 2, Key_buffer2);
    Key_Displayed = true;
  }
}



void Process_Keyboard( void ) {


  if ( KeyIndex == 1 ) {

    Key_Displayed = false;
    Keyboard_Value = Keyboard_Input_Buffer[0];
    KeyIndex = 0;

    // sprintf(b, "Keyboard_Input_Buffer[%02d] =  %0d", OutKeyIndex, Keyboard_Input_Buffer[OutKeyIndex]);
    // Serial.println(b);


    switch ( Keyboard_Mode ) {

      case  ( KEY_CW ):
        Process_CW();
        break;

      case  ( KEY_IDLE ):
        Process_Idle();
        break;

      case  ( KEY_FREQ ):
        Process_Freq();
        break;

      case  ( KEY_TONE ):
        Process_Tone();
        break;

      case  ( KEY_WPM ):
        Process_Speed();
        break;

    }
  }
}



void doMenu(void) {
  int i;
  int Do_Something;

  Current_Menu = Last_Menu;
  updateDisplay();
  Print_Menu_Line();
  lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu], Menu_Cursor_Line_Postions[Current_Menu]);

  //wait for the button to be raised up
  while (Enter_Exit_Down()) delay(50);
  delay(50);  //debounce
  menuOn = 2;

  while (!Enter_Exit_Down() && (menuOn != 0)) {
    Last_Menu = Current_Menu;

    // check if user moving to different menu selection
    i = Menu_Select_Encoder_Read();
    if ( i != 0) {
      if ( (ubitx_mode == MODE_USB) || (ubitx_mode == MODE_LSB) || (ubitx_mode == MODE_SWU) || (ubitx_mode == MODE_SWL)) {
        /*        if (i > 1) {
                  if ( Current_Menu == 0)Current_Menu = 5;
                  else if ( Current_Menu == 7)Current_Menu = 0;
                  else Current_Menu++;
                  lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu], Menu_Cursor_Line_Postions[Current_Menu] );
                } else if (i < -1 ) {
                  if ( Current_Menu == 0)Current_Menu = 7;
                  else if ( Current_Menu == 5)Current_Menu = 0;
                  else Current_Menu--;
                  lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu], Menu_Cursor_Line_Postions[Current_Menu] );
                }
        */
      } else if ( (ubitx_mode == MODE_CW) || (ubitx_mode == MODE_CWR)) {
        if (i > 1) {
          Current_Menu = Current_Menu + 1;
          if (Current_Menu > Max_Menus_Items ) Current_Menu = 0;
          lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu], Menu_Cursor_Line_Postions[Current_Menu] );
        } else if (i < -1 ) {
          Current_Menu = Current_Menu - 1;
          if (Current_Menu < - 0 ) Current_Menu = Max_Menus_Items;
          lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu], Menu_Cursor_Line_Postions[Current_Menu] );
        }
      }
      delay(500);
    } else {

      Do_Something = Test_Button(MENU_CHOOSE_BUTTON, (char*)"Change", (char*)"Exit Menu", (char*)"Exit Menu", (char*)"Exit Menu");
      Print_Menu_Line();
      if ( Do_Something > 1  ) menuOn = 0;
      else if ( Do_Something == 1 ) {
        //else {

        switch  ( Current_Menu ) {

          case  (0):
            menuModeSelect();
            break;
          case  (1):
            menuPaddleSwapToggle();
            break;
          case  (2):
            menuIambicToggle();
            break;
          case  (3):
            menuSetupCwTone();
            break;
          case  (4):
            menuCWSpeed();
            break;
        }
      }
    }
  }
  //Print_Menu_Line();
  //delay(1000);
  printLine2((char *)" ");
  printLine3((char *)" ");
  //printLineXI(2,0,(char *)"  ");
  printLineXI(2, 1, (char *)" ");
  while (Enter_Exit_Down())delay(10);
  updateDisplay();
  LastSmeter = 0;

}



