/**
   ubitx_ui.cpp
   The user interface of the uBITX consists of the encoder, the push-button on top of it
   and the 16x2 LCD display.
   The upper line of the display is constantly used to display frequency and status
   of the radio. Occasionally, it is used to provide a two-line information that is
   quickly cleared up.
*/

//
#include <Arduino.h>
#include "ubitx.h"
#include <LiquidCrystal_Wire2_I2C.h>
extern LiquidCrystal_Wire2_I2C lcd;//  indicate that "lcd" was already defined in <ubitx_V1_08_I2C>
extern void Print_Menu_Line(void);



extern char printBuff[4][21];
extern char c[21], b[21];
extern unsigned long frequency;
extern unsigned long vfoA, vfoB;
extern unsigned long sideTone;    // new menu
extern char inTx;
extern char ritOn;
extern char splitOn;
extern bool isUSB;
extern char vfoActive;
extern int Freq_Underline[6];
extern int Freq_Index;
extern volatile int ubitx_mode;
extern int cwSpeed;
extern int wpm;
extern long ritOffset;
extern bool Volts_Active;
extern unsigned volatile char IAMBIC;
extern unsigned volatile char PDLSWAP;
extern volatile unsigned long Ubitx_Voltage;
extern volatile unsigned long Current_Ubitx_Voltage;
extern volatile unsigned long Ubitx_Voltage;
extern volatile unsigned long Current_Ubitx_Voltage;
extern volatile unsigned long Converted_Ubitx_Voltage;
extern unsigned long Volt_Divider;
extern bool Cat_Lock;
extern bool CAT_S_Meter_Active;
extern bool CW_Calibrated;
extern bool CW_Filter_Present;
extern int Keyboard_Mode;

char Time_Buffer[12];
bool CAT_Volts_Active = false;

int Mem_Number = 0;
int Mem_Table = 3;

char ubitx_mode_text[7][5] = { "USB", "LSB", "CW ", "CWr", "SWU", "SWL" ,"CWP"};
int   Working_Copy_Modes[10];
unsigned long Working_Copy_Frequencies[10];

/**************************************************
   WD9GYM modification for S meter on I2C LCD
   Modified for base LCD code by W8CQD
 **************************************************/
// LCD Bars
byte meter_s1[8]  = {
  B00000,  B00000,  B00000,  B00000,  B00000,  B00000,  B11000,  B11000,
};
byte meter_s3[8]  = {
  B00000,  B00000,  B00000,  B00000,  B00000,  B00011,  B11011,  B11011,
};
byte meter_s5[8]  = {
  B00000,  B00000,  B00000,  B00000,  B11000,  B11000,  B11000,  B11000,
};
byte meter_s7[8]  = {
  B00000,  B00000,  B00000,  B00011,  B11011,  B11011,  B11011,  B11011,
};
byte meter_s9[8]  = {
  B00000,  B00000,  B11000,  B11000,  B11000,  B11000,  B11000,  B11000,
};
byte meter_s10[8] = {
  B00000,  B00011,  B11011,  B11011,  B11011,  B11011,  B11011,  B11011,
};
byte meter_s20[8] = {
  B11000,  B11000,  B11000,  B11000,  B11000,  B11000,  B11000,  B11000,
};
byte meter_s30[8] = {
  B11011,  B11011,  B11011,  B11011,  B11011,  B11011,  B11011,  B11011,
};


volatile int SmeterReadValue, LastSmeter, SMeter_Value, smeterInterval;

/////////////////////////////////////////////////////////////////////////////


// The generic routine to display one partial line on the LCD
void printLineXI(char linenmbr, char index, char *c) {

  uint8_t oldSREG = SREG;
  sei();

  lcd.setCursor(index, linenmbr);             // place the cursor at the selected line
  lcd.print(c);
  //lcd.setCursor(0, linenmbr);             // place the cursor at the beginning of the selected line
  SREG = oldSREG;
}
// The generic routine to display one line on the LCD
void printLine(char linenmbr, char *c) {

  uint8_t oldSREG = SREG;
  sei();

  if (strcmp(c, printBuff[(int)linenmbr])) {     // only refresh the display when there was a change
    lcd.setCursor(0, linenmbr);             // place the cursor at the beginning of the selected line
    lcd.print(c);
    strcpy(printBuff[(int)linenmbr], c);

    for (byte i = strlen(c); i < 20; i++) { // add white spaces until the end of the 16 characters line is reached
      lcd.print(' ');
    }
  }
  SREG = oldSREG;
}

//  short cut to print to the first line
void printLine4(char *c) {
  printLine(3, c);
}

//  short cut to print to the first line
void printLine3(char *c) {
  printLine(2, c);
}

//  short cut to print to the first line
void printLine2(char *c) {
  printLine(1, c);
}

//  short cut to print to the first line
void printLine1(char *c) {
  printLine(0, c);
}


void Place_The_Cursor( void) {
  uint8_t oldSREG = SREG;
  sei();

  if ( ritOn ) {
    lcd.setCursor(18, 2);
  } else if ( !splitOn ) {
    printLineXI(2, 10, (char *)"          ");
    lcd.setCursor(Freq_Underline[Freq_Index], 3);
  } else if ( vfoActive == VFO_B ) lcd.setCursor(Freq_Underline[Freq_Index], 2);
  else lcd.setCursor(Freq_Underline[Freq_Index], 3);
  SREG = oldSREG;

}



//returns true if the button is pressed
bool btnDown(void) {

  if (digitalRead(FBUTTON) == HIGH) return false;
  else return true;
}

bool Enter_Exit_Down(void) {

  if (digitalRead(MENU_ENTER_BUTTON) == HIGH) return false;
  else return true;

}

bool Choose_Down(void) {

  if (digitalRead(MENU_CHOOSE_BUTTON) == HIGH) return false;
  else return true;
}


void Display_S_Meter( void ) {
  char SmeterText[16];
  uint8_t oldSREG;

  int  level;

  if ( Keyboard_Mode != KEY_CW ) {
    oldSREG = SREG;
    sei();
    SmeterReadValue = analogRead(SMETERREADPIN);   // read value of signal 0 - 1023
    level = SmeterReadValue;  // kill time
    SmeterReadValue = analogRead(SMETERREADPIN);   // read value of signal 0 - 1023
    SREG = oldSREG;



    level = SmeterReadValue / 25;

    if (level >= 30) level = 40;
    if (level >= 20 && level < 30) level = 30;
    if (level >= 10 && level < 20) level = 20;
    if (level >= 7 && level < 10) level = 9;
    if (level >= 5 && level < 8) level = 7;
    if (level >= 3 && level < 6) level = 5;
    if (level >= 1 && level < 4) level = 3;
    if (level >= 0 && level < 2) level = 1;
    if ( Cat_Lock == false ) {
      if (level != LastSmeter ) {
        sprintf(SmeterText, "s       ");
        printLineXI(1, 12, SmeterText);                     // blank out bars
        lcd.createChar(4, meter_s30);         // let the code fall through
        switch ( level ) {                         // write each bar required
          case 40:                                 // do not put break between case statements
            //lcd.setCursor(19, 1);
            lcd.setCursor(17, 1);
            lcd.write( 4 );
          case 30:
          case 20:
            if ( level == 20 ) lcd.createChar(4, meter_s20);
            //lcd.setCursor(18, 1);
            lcd.setCursor(16, 1);
            lcd.write( 4 );
          case 10:
            lcd.createChar(3, meter_s10);
          case 9:
            if ( level == 9 ) lcd.createChar(3, meter_s9);
            //lcd.setCursor(17, 1);
            lcd.setCursor(15, 1);
            lcd.write( 3 );
          case 7:
            lcd.createChar(2, meter_s7);
          case 5:
            if ( level == 5 ) lcd.createChar(2, meter_s5);
            //lcd.setCursor(16, 1);
            lcd.setCursor(14, 1);
            lcd.write( 2 );
          case 3:
            lcd.createChar(1, meter_s3);
          case 1:
            if ( level == 1 )lcd.createChar(1, meter_s1);
            //lcd.setCursor(15, 1);
            lcd.setCursor(13, 1);
            lcd.write( 1 );
          case 0:
          default:
            break;
        }
      }
    } else {
      if ( CAT_S_Meter_Active == true ) {
        sprintf(SmeterText, "uMV%02d;", (int)level);
        Serial.write(SmeterText);
        delay(10);
      }
    }
    LastSmeter = level;
    Place_The_Cursor();

  }
}


void Display_Voltage( void ) {
  char local_b[12];
  int i;

  for (i = 0; i < 12; i++)Time_Buffer[i] = 0;
  // Display VCC volts
  if ( (Volts_Active) && (!inTx )) {
    Converted_Ubitx_Voltage = (Ubitx_Voltage * (unsigned long)150);
    Converted_Ubitx_Voltage = Converted_Ubitx_Voltage  / Volt_Divider;
    ultoa(Converted_Ubitx_Voltage, local_b, 10);
    if ( strlen(local_b) == 1) {
      strcpy(Time_Buffer, "00");
      strcat(Time_Buffer, ".");
      strncat(Time_Buffer, b, 1);
    } else if ( strlen(local_b) == 2) {
      strcpy(Time_Buffer, "0");
      strncat(Time_Buffer, local_b, 1);
      strcat(Time_Buffer, ".");
      strncat(Time_Buffer, &local_b[1], 1);
    } else {
      strncpy(Time_Buffer, local_b, 2);
      strcat(Time_Buffer, ".");
      strncat(Time_Buffer, &local_b[2], 1);
    }
    strcat(Time_Buffer, "V ");
    Time_Buffer[strlen(Time_Buffer)] = 0;
    Current_Ubitx_Voltage = Ubitx_Voltage;
    printLineXI(3, 0, Time_Buffer);
  } else printLineXI(3, 0, (char *)"     ");
  if ( Cat_Lock && CAT_Volts_Active ) {

    sprintf(local_b, "uVR%s;", Time_Buffer);
    local_b[strlen(local_b)] = 0;
    Serial.write(local_b);
    delay(10);

  }
}

void Display_Mode( void ) {

  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));

  if ( ubitx_mode == MODE_USB )       strcpy(c, "USB");
  else if ( ubitx_mode == MODE_LSB   )strcpy(c, "LSB");
  else if ( ubitx_mode == MODE_CW    )strcpy(c, " CW");
  else if ( ubitx_mode == MODE_CWR   )strcpy(c, "CWr");
  else if ( ubitx_mode == MODE_SWU   )strcpy(c, "SWU");
  else if ( ubitx_mode == MODE_SWL   )strcpy(c, "SWL");
  else if ( ubitx_mode == MODE_CWP   )strcpy(c, "CWP");
  if (vfoActive == VFO_A) {
    printLineXI(2, 6, (char *)"   ");
    printLineXI(3, 6, c);
  } else if ( splitOn ) {
    printLineXI(3, 6, (char *)"   ");
    printLineXI(2, 6, c);
  } else {
    printLineXI(2, 6, (char *)"   ");
    printLineXI(3, 6, c);
  }

}

void Display_RIT( void) {
  unsigned long temp;

  if ( ritOffset < 0 ) {
    strcpy(c, "- ");
    temp = 0 - ritOffset;
  } else {
    strcpy(c, "+ ");
    temp = ritOffset;
  }
  ultoa(temp / 100, b, DEC);
  if ( strlen(b) == 1) {
    strcat(c, "0.");
    strncat(c, &b[0], 1);
  }
  if ( strlen(b) == 2) {
    strncat(c, &b[0], 1);
    strcat(c, ".");
    strncat(c, &b[1], 1);
  }
  strcat(c, " ");

  printLineXI(2, 14, c);
}

char Memory_Tables[4][4] = {"CW ", "SSB", "DIG", "USR"};
extern int Memory_Table_Select;

void Display_Mem_Selection( void ) {
  char fred[10];

  if ( !Cat_Lock ) {

    sprintf(fred, "%3s:%1d   ", Memory_Tables[Memory_Table_Select], Mem_Number);

    printLineXI(0, 0, fred);
    printLineXI(0, 6, ubitx_mode_text[Working_Copy_Modes[Mem_Number]]);
    memset(c, 0, sizeof(c));
    memset(b, 0, sizeof(b));
    ultoa(Working_Copy_Frequencies[Mem_Number], b, DEC);
    if (strlen(b) < 7) {
      strcat(&c[0], "00.");
      strncat(c, &b[0], 3);
      strcat(c, ".");
      strncat(c, &b[3], 2);
    } else if (strlen(b) < 8) {
      strcat(&c[0], "0");
      strncat(c, &b[0], 1);
      strcat(c, ".");
      strncat(c, &b[1], 3);
      strcat(c, ".");
      strncat(c, &b[4], 2);
    } else {
      strncat(c, b, 2);
      strcat(c, ".");
      strncat(c, &b[2], 3);
      strcat(c, ".");
      strncat(c, &b[5], 2);
    }

    printLineXI(1, 0, c);
  }
}




void updateDisplay(void) {



  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));



  if ( Cat_Lock ) {
    printLineXI(0, 0, (char*)"         ");
    Print_Menu_Line();
  }

  //now, the third line
  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));


  if (splitOn) {
    strcpy(c, "T");
  } else {
    if (vfoActive == VFO_A) strcpy(c, "b");
    else strcpy(c, "B");
  }
  ultoa(vfoB, b, DEC);
  if (strlen(b) < 7) {
    strcat(&c[1], "00.");
    strncat(c, &b[0], 3);
    strcat(c, ".");
    strncat(c, &b[3], 2);
  } else if (strlen(b) < 8) {
    strcat(&c[1], "0");
    strncat(c, &b[0], 1);
    strcat(c, ".");
    strncat(c, &b[1], 3);
    strcat(c, ".");
    strncat(c, &b[4], 2);
  } else {
    strncat(c, b, 2);
    strcat(c, ".");
    strncat(c, &b[2], 3);
    strcat(c, ".");
    strncat(c, &b[5], 2);
  }

  if ( splitOn ) printLineXI(2, 10, c);
  else if ( vfoActive == VFO_A ) {
    if (!ritOn)printLineXI(2, 10, (char *)"          ");
  }
  else printLineXI(3, 10, c);

  Display_Mode();

  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));



  if (splitOn) {
    strcat(c, "R");
  } else {
    strcat(c, "A");
  }
  ultoa(vfoA, b, DEC);
  if (strlen(b) < 7) {
    strcat(&c[1], "00.");
    strncat(c, &b[0], 3);
    strcat(c, ".");
    strncat(c, &b[3], 2);
  } else if (strlen(b) < 8) {
    strcat(&c[1], "0");
    strncat(c, &b[0], 1);
    strcat(c, ".");
    strncat(c, &b[1], 3);
    strcat(c, ".");
    strncat(c, &b[4], 2);
  } else {
    strncat(c, b, 2);
    strcat(c, ".");
    strncat(c, &b[2], 3);
    strcat(c, ".");
    strncat(c, &b[5], 2);
  }

  if ( splitOn ) printLineXI(3, 10, c);
  else {
    //printLineXI(2, 0, (char*)"          ");
    if ( vfoActive == VFO_A ) printLineXI(3, 10, c);
  }



  if (ritOn) Display_RIT();

  if ( (ubitx_mode == MODE_CW) || (ubitx_mode == MODE_CWR)) {
    if (!CW_Calibrated && CW_Filter_Present)printLineXI(2, 0, (char*)"*CW CAL   ");
  }

  Place_The_Cursor();


}

int enc_prev_state = 3;
int Menu_Select_enc_prev_state = 3;
int DSP_Center_enc_prev_state = 3;
int DSP_Width_enc_prev_state = 3;

/**
   The A0 And A1 are purely analog lines on the Arduino Nano
   These need to be pulled up externally using two 10K resistors

   There are excellent pages on the Internet about how these encoders work
   and how they should be used. We have elected to use the simplest way
   to use these encoders without the complexity of interrupts etc to
   keep it understandable.

   The enc_state returns a two-bit number such that each bit reflects the current
   value of each of the two phases of the encoder

   The enc_read returns the number of net pulses counted over 50 msecs.
   If the puluses are -ve, they were anti-clockwise, if they are +ve, the
   were in the clockwise directions. Higher the pulses, greater the speed
   at which the enccoder was spun
*/


int enc_read( void ) {
  int result = 0;
  byte newState;
  int enc_speed = 0;

  unsigned long previousMillis = millis();
  unsigned long stop_by = 50;
  unsigned long currentMillis = millis();

  while ((currentMillis - previousMillis) < stop_by) { // check if the previous state was stable
    newState = (digitalRead(ENC_A) == HIGH ? 1 : 0) + (digitalRead(ENC_B) == HIGH ? 2 : 0); // Get current state

    if (newState != enc_prev_state)
      delay (1);

    if ( (digitalRead(ENC_A) == HIGH ? 1 : 0) + (digitalRead(ENC_B) == HIGH ? 2 : 0) != newState || newState == enc_prev_state) {
      currentMillis = millis();
      continue;
    } else {
      //these transitions point to the encoder being rotated anti-clockwise
      if ((enc_prev_state == 0 && newState == 2) ||
          (enc_prev_state == 2 && newState == 3) ||
          (enc_prev_state == 3 && newState == 1) ||
          (enc_prev_state == 1 && newState == 0)) {
        //result--;
        result++;
      }
      //these transitions point o the enccoder being rotated clockwise
      if ((enc_prev_state == 0 && newState == 1) ||
          (enc_prev_state == 1 && newState == 3) ||
          (enc_prev_state == 3 && newState == 2) ||
          (enc_prev_state == 2 && newState == 0)) {
        //result++;
        result--;
      }
      enc_prev_state = newState; // Record state for next pulse interpretation
      enc_speed++;
      delay(1);
      currentMillis = millis();
    }
  }


  return (result);
}



int Menu_Select_Encoder_Read(void) {

  int result = 0;
  byte newState;
  int enc_speed = 0;

  unsigned long stop_by = millis() + 50;

  while (millis() < stop_by) { // check if the previous state was stable
    newState = (digitalRead(MENU_SEL_ENC_A) == true ? 1 : 0) + (digitalRead(MENU_SEL_ENC_B) == true ? 2 : 0); // Get current state

    if (newState != Menu_Select_enc_prev_state)
      delay (1);

    if ( (digitalRead(MENU_SEL_ENC_A) == true ? 1 : 0) + (digitalRead(MENU_SEL_ENC_B) == true ? 2 : 0) != newState || newState == Menu_Select_enc_prev_state)
      continue;
    //these transitions point to the encoder being rotated anti-clockwise
    if ((Menu_Select_enc_prev_state == 0 && newState == 2) ||
        (Menu_Select_enc_prev_state == 2 && newState == 3) ||
        (Menu_Select_enc_prev_state == 3 && newState == 1) ||
        (Menu_Select_enc_prev_state == 1 && newState == 0)) {
      result--;
    }
    //these transitions point o the enccoder being rotated clockwise
    if ((Menu_Select_enc_prev_state == 0 && newState == 1) ||
        (Menu_Select_enc_prev_state == 1 && newState == 3) ||
        (Menu_Select_enc_prev_state == 3 && newState == 2) ||
        (Menu_Select_enc_prev_state == 2 && newState == 0)) {
      result++;
    }
    Menu_Select_enc_prev_state = newState; // Record state for next pulse interpretation
    enc_speed++;
    delay(1);
  }


  return (result);
}





